@@ -10,8 +10,8 @@ This currently supports:
  * Python modules (./*.py, only in root directory)
  * Python packages (all directories with __init__.py)
  * Docbook-XML GNOME help files (help/<language>/{*.xml,*.omf,figures})
- * GtkBuilder (*.ui) [installed into prefix/share/<projectname>/]
- * Qt4 user interfaces (*.ui) [compiled with pykdeuic into Python modules]
+ * GtkBuilder and Qt4 user interfaces (*.ui) [installed into
+   prefix/share/<projectname>/]
  * D-Bus (*.conf and *.service)
  * PolicyKit (*.policy.in)
  * Desktop files (*.desktop.in) [into prefix/share/applications, or
@@ -40,6 +40,7 @@ author, license, etc.) in ./setup.py.
 import os, os.path, fnmatch, stat, sys, subprocess
 import ast
 import distutils.core
+from functools import reduce
 
 from DistUtilsExtra import __version__ as __pkgversion
 from DistUtilsExtra.command import *
@@ -88,7 +89,7 @@ def setup(**attrs):
     __data(attrs, src)
     __scripts(attrs, src)
     __stdfiles(attrs, src)
-    __gtkbuilder(attrs, src)
+    __ui(attrs, src)
     __manpages(attrs, src)
 
     if 'clean' not in sys.argv:
@@ -97,9 +98,9 @@ def setup(**attrs):
     distutils.core.setup(**attrs)
 
     if src:
-        print 'WARNING: the following files are not recognized by DistUtilsExtra.auto:'
+        print('WARNING: the following files are not recognized by DistUtilsExtra.auto:')
         for f in sorted(src):
-            print ' ', f
+            print ('  ' + f)
 
 #
 # parts of setup()
@@ -123,17 +124,17 @@ def __cmdclass(attrs):
     v.setdefault('build_help', build_help_auto)
     v.setdefault('build_i18n', build_i18n_auto)
     v.setdefault('build_icons', build_icons.build_icons)
-    v.setdefault('build_kdeui', build_kdeui_auto)
     v.setdefault('install', install_auto)
     v.setdefault('clean', clean_build_tree)
     v.setdefault('sdist', sdist_auto)
+    v.setdefault('pylint', pylint.pylint)
 
 def __modules(attrs, src):
     '''Default modules'''
 
     if 'py_modules' in attrs:
         for mod in attrs['py_modules']:
-            print mod
+            print(mod)
             src_markglob(src, os.path.join(mod, '*.py'))
         return
 
@@ -222,7 +223,7 @@ def __data(attrs, src):
     data_files = []
     for f in src.copy():
         if f.startswith('data/') and not f.startswith('data/icons/') and \
-                not f.endswith('.desktop.in') and not f.endswith('*.notifyrc.in'):
+                not f.endswith('.desktop.in') and not f.endswith('.notifyrc.in'):
             if not os.path.islink(f):
                 # symlinks are handled in install_auto
                 v.append((os.path.join('share', attrs['name'], os.path.dirname(f[5:])), [f]))
@@ -276,13 +277,16 @@ def __stdfiles(attrs, src):
         attrs.setdefault('data_files', []).append((os.path.join('share', 'doc',
             attrs['name']), readme))
 
-def __gtkbuilder(attrs, src):
-    '''Install GtkBuilder *.ui files'''
+def __ui(attrs, src):
+    '''Install GtkBuilder/Qt *.ui files'''
 
     ui = []
     for f in src_fileglob(src, '*.ui'):
-        contents = open(f).read()
-        if ('<interface>\n' in contents or '<interface ' in contents) and 'class="Gtk' in contents:
+        fd = open(f)
+        firstlines = fd.readline()
+        firstlines += '\n' + fd.readline()
+        fd.close()
+        if '<interface' in firstlines or '<ui version=' in firstlines:
             src_mark(src, f)
             ui.append(f)
     if ui:
@@ -301,7 +305,7 @@ def __manpages(attrs, src):
             src_mark(src, f)
             mans.setdefault(f[-1], []).append(f)
     v = attrs.setdefault('data_files', [])
-    for section, files in mans.iteritems():
+    for section, files in mans.items():
         v.append((os.path.join('share', 'man', 'man' + section), files))
 
 def __external_mod(module, attrs):
@@ -317,7 +321,7 @@ def __external_mod(module, attrs):
     try:
         path = __import__(module).__file__
     except ImportError:
-        print >> sys.stderr, 'ERROR: Python module %s not found' % module
+        sys.stderr.write('ERROR: Python module %s not found\n' % module)
         return False
     except AttributeError: # builtin modules
         return False
@@ -342,7 +346,7 @@ def __add_imports(imports, file, attrs):
                 if __external_mod(node.module, attrs):
                     imports.add(node.module)
     except SyntaxError, e:
-        print >> sys.stderr, 'WARNING: syntax errors in', file, ':', e
+        sys.stderr.write('WARNING: syntax errors in %s: %s\n' % (file, str(e)))
 
 def _module_parents(mod):
     '''Iterate over all parents of a module'''
@@ -402,7 +406,7 @@ def __provides(attrs, src_all):
     if 'provides' in attrs:
         return
 
-    provides = attrs.get('py_modules', [])
+    provides = list(attrs.get('py_modules', [])) # we need a copy here
     for p in attrs.get('packages', []):
         provides.append(p.replace(os.path.sep, '.'))
     attrs['provides'] = __filter_namespace(provides)
@@ -575,9 +579,9 @@ class build_i18n_auto(build_i18n.build_i18n):
                     if not os.path.isdir('po'):
                         os.mkdir('po')
                     potfiles_in = open('po/POTFILES.in', 'w')
-                    print >> potfiles_in, '[encoding: UTF-8]'
+                    potfiles_in.write('[encoding: UTF-8]\n')
                     for f in files:
-                        print >> potfiles_in, f
+                        potfiles_in.write(f + '\n')
                     potfiles_in.close()
 
                     auto_potfiles_in = True
@@ -594,30 +598,6 @@ class build_i18n_auto(build_i18n.build_i18n):
             except:
                 pass
 
-class build_kdeui_auto(build_kdeui.build_kdeui):
-    def finalize_options(self):
-        global src
-
-        # add *.ui files which belong to KDE4
-        kdeui_files = []
-        for f in src_fileglob(src, '*.ui'):
-            fd = open(f)
-            # might be on the first or second line
-            if fd.readline().startswith('<ui version="') or \
-               fd.readline().startswith('<ui version="'):
-                src_mark(src, f)
-                kdeui_files.append(f)
-            fd.close()
-        if kdeui_files:
-            try:
-                uf = eval(self.ui_files)
-            except TypeError:
-                uf = []
-            uf += kdeui_files
-            self.ui_files = repr(uf)
-
-        build_kdeui.build_kdeui.finalize_options(self)
-
 #
 # Automatic sdist
 #
